#!/usr/local/bin/pyhon3
'''
    Using DiceSimulator.py for the rendering
    http://python3.codes/a-graphical-dice-simulator/

    And pygame/examples/scaletest.py for the conrol loop 
    https://www.pygame.org/

    Author: Marco Mambelli
    
'''

import pygame 
import random
import time


# Constants
DSIZE = 256                      # Size of window/dice
spsz = DSIZE//10                 # size of spots
m = int(DSIZE/2)                 # mid-point of dice (or die?)
l=t=int(DSIZE/4)                 # location of left and top spots
r=b=DSIZE-l                      # location of right and bottom spots
rolling = 12                    # times that dice rolls before stopping
DEFAULT_DICECOL = (255,255,127)          # die colour
DEFAULT_SPOCOL = (0,127,127)           # spot colour
 

class Dice:
    IMPOSSIBLE_TRANSITIONS = [ 0, 6, 5, 4, 3, 2, 1 ] # 0 added for completion n+new_n=7
    def __init__(self, display, n=1, diecol=DEFAULT_DICECOL, spotcol=DEFAULT_SPOCOL):
        self.d = display
        self.diecol = diecol
        self.spotcol = spotcol
        self.n = n

    def paint(self, n):
        """ Change the value of dice and paint it"""
        self.n = n
        self.d.fill(self.diecol)                          # clear previous spots
        if n % 2 == 1:
            pygame.draw.circle(self.d,self.spotcol,(m,m),spsz)  # middle spot
        if n==2 or n==3 or n==4 or n==5 or n==6:
            pygame.draw.circle(self.d,self.spotcol,(l,b),spsz)  # left bottom
            pygame.draw.circle(self.d,self.spotcol,(r,t),spsz)  # right top
        if n==4 or n==5 or n==6:
            pygame.draw.circle(self.d,self.spotcol,(l,t),spsz)  # left top
            pygame.draw.circle(self.d,self.spotcol,(r,b),spsz)  # right bottom
        if n==6:
            pygame.draw.circle(self.d,self.spotcol,(m,b),spsz)  # middle bottom
            pygame.draw.circle(self.d,self.spotcol,(m,t),spsz)  # middle top
        pygame.display.flip()

    def roll(self, roll_max=20):
        """Roll the dice (multiple changes)"""
        for i in range(int(random.random() * roll_max)):
            n = random.randint(1,6)
            if self.n + n == 7:  # impossible ransition
                continue
            self.paint(n)
            time.sleep(0.2)


def main():
    """show dice - main loop
    """
    # pygame.init()
    # initialize display
    pygame.display.init()
    d = pygame.display.set_mode((DSIZE, DSIZE))
    d.fill(DEFAULT_DICECOL)
    pygame.display.set_caption("Dice Simulator")
    # Add text
    pygame.font.init()
    myfont = pygame.font.SysFont("monospace", 15)
    label = myfont.render("Press UP to roll!", 1, DEFAULT_SPOCOL)
    label2 = myfont.render("and ESC o Quit", 1, DEFAULT_SPOCOL)
    d.blit(label, (20, 20))
    d.blit(label2, (20, 40))
    # turn off the mouse pointer
    # pygame.mouse.set_visible(0)
    # Get the dice
    dice = Dice(d)
    # main loop
    bRunning = True
    bUp = False
    bDown = False
    bLeft = False
    bRight = False
    while(bRunning):
        pygame.display.flip()
        for event in pygame.event.get():
            if event.type == pygame.QUIT or (event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE):
                bRunning = False
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_UP:    bUp = True
                if event.key == pygame.K_DOWN:  bDown = True
                if event.key == pygame.K_LEFT:  bLeft = True
                if event.key == pygame.K_RIGHT: bRight = True
            if event.type == pygame.KEYUP:
                if event.key == pygame.K_UP:    bUp = False
                if event.key == pygame.K_DOWN:  bDown = False
                if event.key == pygame.K_LEFT:  bLeft = False
                if event.key == pygame.K_RIGHT: bRight = False
        if bUp:
            dice.roll()


if __name__ == "__main__":
    main()
