import sys
import wave
import math
import struct
import random
import argparse
from itertools import *


def damped_wave(frequency=440.0, framerate=44100, amplitude=0.5, length=1):
    if amplitude > 1.0: amplitude = 1.0
    if amplitude < 0.0: amplitude = 0.0
    return (math.exp(-(float(i%(length*44100))/float(framerate))) * s for i, s in enumerate(sine_wave(frequency, framerate, amplitude)))


def ncycles(iterable, n):
    #"Returns the sequence elements n times"
    return chain.from_iterable(repeat(tuple(iterable), n))

def sine_wave(frequency, framerate=44100, amplitude=0.5):
    period = int(framerate / frequency)
    if amplitude > 1.0: amplitude = 1.0 #amplitude affects location of sound (experiment with later)
    if amplitude < 0.0: amplitude = 0.0
    lookup_table = [float(amplitude) * math.sin(2.0*math.pi*float(frequency)*(float(i%period)/float(framerate))) for i in range(period)] #calculates one period of the sound
    return (lookup_table[i%period] for i in count(0))

def white_noise(amplitude=0.5): #completely random sound data
    return (float(amplitude) * random.uniform(-1, 1) for _ in count(0))

def compute_samples(channels, nsamples=None): #computes the total wave of all channels
    return islice(zip(*(map(sum, zip(*channel)) for channel in channels)), nsamples)

def grouper(n, iterable, fillvalue=None):
    #"grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx"
    args = [iter(iterable)] * n
    return zip_longest(*args, fillvalue=fillvalue)

#nchannels = number of channels (1=mono; 2=stereo)
#sampwidth = size of each sample(16 bit = 2)
#framerate - the number of samples per second
#nframes - framerate*duration of file(in seconds)
def write_wavefile(filename, samples, nframes=None, nchannels=2, sampwidth=2, framerate=44100, bufsize=2048):
    if nframes is None:
        nframes = -1

    w = wave.open(filename, 'w')
    w.setparams((nchannels, sampwidth, framerate, nframes, 'NONE', 'not compressed'))

    max_amplitude = float(int((2 ** (sampwidth * 8)) / 2) - 1)

    # split the samples into chunks (to reduce memory consumption and improve performance)
    for chunk in grouper(bufsize, samples):
        frames = b''.join(b''.join(struct.pack('h', int(max_amplitude * sample)) for sample in channels) for channels in chunk if channels is not None)
        w.writeframesraw(frames)
    w.close()
    return filename

