#Make a series of notes
#Write wav song file
import math
import random

noteFreq = [16.35, 17.32, 18.35, 19.45, 20.6, 21.83, 23.12, 24.5, 25.96, 27.5, 29.14, 30.87] #From C0-B0
noteID = {'C': 0, 'D': 2, 'E': 4, 'F': 5, 'G': 7, 'A': 9, 'B': 11}
maj = [2,2,1,2,2,2,1]
min = [2,1,2,2,1,2,2]
intervals = [3,4,5]
noteLen = []
MAX_OCTAVE = 5
MIN_OCTAVE = 3


def getScale(key):
    strLen = len(key)
    scale = []
    #Get index of tonic
    note = noteID[key[0]]
    note-=12
    #Check if sharp or flat
    if strLen > 1:
        aDental = key[1]
        if aDental == 'b': #If flat, go down 1 half step
            note-=1
        elif aDental == '#': #If sharp, go up 1 half step
            note+=1
    if note >= 11: #Make sure note is within noteFreq limits [-12,11]
        note-=12
    elif note <= -13:
        note+=12
    if key[strLen-1] == 'm': #If key is minor, use minor set
        for i in min:
            scale.append(noteFreq[note])
            note+=i
    else: #If key is major, use major set
        for i in maj:
            scale.append(noteFreq[note])
            note+=i
    return scale #returns an array of valid notes

def getFreq(freq, octave):
    return freq*(2**octave) #returns frequency of pitch in an octave

def setNoteLen(timeSign, tempo):
    noteLen = [0.0,1.0,2.0,3.0]
    beat = float(60/tempo) #length of one beat (in seconds)
    #strBreak = timeSign.index("/")
    #bottom = int(timeSign[strBreak+1 : len(timeSign)]) #denominator (bottom)
    base = 2 #math.log(bottom,2) #index of note that is one beat (2 = half, 4 = quarter, etc.)
    for i in range(0,4):
        noteLen[i] = round(beat*(2**(base-i)),2) #initialize length of each note type in seconds
    return noteLen #0 = whole, 1 = half, 2 = quarter, 3 = eight, 4 = sixteenth

def getMeasureSum(timeSign, tempo): #returns length of measure in seconds
    strBreak = timeSign.index("/") #split str tempo
    topNum = int(timeSign[0 : strBreak]) #numerator (topNum)
    measure = round(float(topNum*60/tempo),2)
    return measure

def getRhythms(measureSum, songLen, noteLen): #songLen = number of measures in song
    rhythm = [] #array of rhythms for the whole song
    for i in range(songLen):
        sum = 0;
        while sum < measureSum:
            index = int(random.random()*4)
            note = round(noteLen[index],2)
            if note+sum <= measureSum:
                sum+=note
                sum = round(sum,2)
                rhythm.append(note)
            if measureSum-sum<noteLen[len(noteLen)-1]:
                break;
    return rhythm #return full rhythm for each song

def getPitches(rhythms, scale):
    pitches = []
    octave = 4
    lastNote = None
    for time in rhythms: #for every time in rhythms
        if lastNote == None: #if is first note
            #freq = []
            note = int(random.random() * 7) #choose a random note from scale
        else:
            choice = random.random()*2;
            if choice <= 0.5: #go up/down a step
                choice = random.random()*2
                if choice <= 0.5: #go down
                    note-=1
                    if note<0:
                        note+=7
                        if octave>MIN_OCTAVE: #if Do
                            octave-=1
                else: #go up
                    note+=1
                    if note>6:
                        note-=7
                        if octave < MAX_OCTAVE: #if Ti
                            octave+=1
            else: #go an interval
                choice = random.random()*2
                i = int(random.random()*3)
                interval = intervals[i]-1
                if choice <=0.5: #go down
                    note-=interval
                    if note<0:
                        note+=7
                        if octave>MIN_OCTAVE:
                            octave-=1
                else: #go up
                    note+=interval
                    if note>6:
                        note-=7
                        if octave<MAX_OCTAVE:
                            octave+=1
        pitch = getFreq(scale[note], octave) #pitch = frequency of note
        #freq.append(pitch) #add each pitch to freq
        pitches.append(pitch) #add freq to pitches
        lastNote = note;
    return pitches;

